local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
if not HeadCount then return end

local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")

HeadCount.DateFormats = {
	[1] = "Day-Month-Year",
	[2] = "Month-Day-Year",
	[3] = "Year-Day-Month",
}

local partyGroups = { "1", "2", "3", "4", "5", "6", "7", "8", }
local exportFormats = {
	["EQdkp"] = L["EQdkp"],
	["DKPBoard"] = L["DKPBoard"],
	["phpBB"] = L["phpBB"],
	["phpBB_ItemStats"] = L["phpBB_ItemStats"],
	["CSV"] = L["CSV"],
	["Text"] = L["Text"],
	["XML"] = L["XML"],
}
local channels = {
	["guild"] = "GUILD",
	["officer"] = "OFFICER",
	["party"] = "PARTY",
	["raid"] = "RAID",
	["say"] = "SAY",
	["yell"] = "YELL",
}

-- configuration options
local options = {
    type = "group",
	handler = HeadCount,
    args = {
        gui = {
            type = "execute",
            name = L["console.command.gui.name"],
            desc = L["console.command.gui.description"],
			func = "ToggleUI",
			order = 1,
        },
		raid = {
			type = "group",
			name = L["console.command.raid.name"],
			desc = L["console.command.raid.description"],
			order = 2,
		    args = {
				groupsetup = {
					type = "group",
					name = L["console.command.raid.groupsetup.name"],
					desc = L["console.command.raid.groupsetup.description"],
					order = 1,
					args = {
						raidlistgroups = {
							type = "input",
							name = L["console.command.raid.groupsetup.raidlistgroups.name"],
							desc = L["console.command.raid.groupsetup.raidlistgroups.description"],
							usage = L["console.usage.groups"],
							multiToggle = true,
							get = "GetRaidListGroups",
							set = function(key, value)
									HeadCount:SetRaidListGroups(key, value)
									HeadCount:RAID_ROSTER_UPDATE(false)
							end,
							validate = partyGroups,
							order = 1,
						},
						waitlistgroups = {
							type = "input",
							name = L["console.command.raid.groupsetup.waitlistgroups.name"],
							desc = L["console.command.raid.groupsetup.waitlistgroups.description"],
							usage = L["console.usage.groups"],
							multiToggle = true,
							get = "GetWaitListGroups",
							set = function(key, value)
									HeadCount:SetWaitListGroups(key, value)
									HeadCount:RAID_ROSTER_UPDATE(false)
							end,
							validate = partyGroups,
							order = 2,
						},
						autogrouping = {
							type = "toggle",
							name = L["console.command.raid.groupsetup.autogrouping.name"],
							desc = L["console.command.raid.groupsetup.autogrouping.description"],
							get = "IsAutomaticGroupSelectionEnabled",
							set = "ToggleAutomaticGroupSelection",
							order = 3,
						},
					},
				},
				waitlist = {
					type = "group",
					name = L["console.command.raid.waitlist.name"],
					desc = L["console.command.raid.waitlist.description"],
					order = 2,
					args = {
						duration = {
							type = "range",
							name = L["console.command.raid.waitlist.duration.name"],
							desc = L["console.command.raid.waitlist.duration.description"],
							get = "GetWaitlistDuration",
							set = function(duration)
								HeadCount:SetWaitlistDuration(duration)
								HeadCount:manageWaitlistChange(duration)
							end,
							min = 0,
							max = 30,
							step = 1,
							order = 1,
						},
						autoremoval = {
							type = "toggle",
							name = L["console.command.raid.waitlist.autoremoval.name"],
							desc = L["console.command.raid.waitlist.autoremoval.description"],
							get = "IsWaitlistAutoremovalEnabled",
							set = function(key, value)
								HeadCount:ToggleWaitlistAutoremoval()
								HeadCount:RAID_ROSTER_UPDATE(false)
							end,
							order = 2,
						},
						incoming = {
							type = "toggle",
							name = L["console.command.raid.waitlist.incoming.name"],
							desc = L["console.command.raid.waitlist.incoming.description"],
							get = "IsWaitlistIncomingEnabled",
							set = "ToggleWaitlistIncoming",
							order = 3,
						},
						outgoing = {
							type = "toggle",
							name = L["console.command.raid.waitlist.outgoing.name"],
							desc = L["console.command.raid.waitlist.outgoing.description"],
							get = "IsWaitlistOutgoingEnabled",
							set = "ToggleWaitlistOutgoing",
							order = 4,
						},
						notify = {
							type = "toggle",
							name = L["console.command.raid.waitlist.notify.name"],
							desc = L["console.command.raid.waitlist.notify.description"],
							get = "IsWaitlistNotifyEnabled",
							set = "ToggleWaitlistNotify",
							order = 5,
						}
					},
				},
				pruning = {
					type = "group",
					name = L["console.command.raid.pruning.name"],
					desc = L["console.command.raid.pruning.description"],
					order = 3,
					args = {
						prune = {
							type = "toggle",
							name = L["console.command.raid.pruning.prune.name"],
							desc = L["console.command.raid.pruning.prune.description"],
							get = "IsPruningEnabled",
							set = "TogglePruning",
							order = 1,
						},
						prunetime = {
							type = "range",
							name = L["console.command.raid.pruning.prunetime.name"],
							desc = L["console.command.raid.pruning.prunetime.description"],
							get = "GetPruningTime",
							set = "SetPruningTime",
							min = 1,
							max = 52,
							step = 1,
							order = 2,
						},
					},
				},
				delay = {
					type = "range",
					name = L["console.command.raid.delay.name"],
					desc = L["console.command.raid.delay.description"],
					get = "GetDelay",
					set = "SetDelay",
					min = 0,
					max = 20,
					step = 1,
					order = 4,
				},
				bgtracking = {
					type = "toggle",
					name = L["console.command.raid.bgtracking.name"],
					desc = L["console.command.raid.bgtracking.description"],
					get = "IsBattlegroundTrackingEnabled",
					set = "ToggleBattlegroundTracking",
					order = 5,
				},
			},
		},
		["datetime"] = {
			type = "group",
			name = L["console.command.datetime.name"],
			desc = L["console.command.datetime.description"],
			order = 3,
			args = {
			    dateformat = {
					type = "input",
					name = L["console.command.datetime.dateformat.name"],
					desc = L["console.command.datetime.dateformat.description"],
					usage = L["console.usage.dateformat"],
					get = "GetDateFormat",
					set = "SetDateFormat",
					validate = L.dateFormatting,
					order = 1,
				},
				timetotals = {
					type = "group",
					name = L["console.command.datetime.timetotals.name"],
					desc = L["console.command.datetime.timetotals.description"],
					order = 2,
					args = {
						raidlisttime = {
							type = "toggle",
							name = L["console.command.datetime.timetotals.raidlisttime.name"],
							desc = L["console.command.datetime.timetotals.raidlisttime.description"],
							get = "IsRaidListTimeEnabled",
							set = "ToggleRaidListTime",
							order = 1,
						},
						waitlisttime = {
							type = "toggle",
							name = L["console.command.datetime.timetotals.waitlisttime.name"],
							desc = L["console.command.datetime.timetotals.waitlisttime.description"],
							get = "IsWaitListTimeEnabled",
							set = "ToggleWaitListTime",
							order = 2,
						},
						offlinetime = {
							type = "toggle",
							name = L["console.command.datetime.timetotals.offlinetime.name"],
							desc = L["console.command.datetime.timetotals.offlinetime.description"],
							get = "IsOfflineTimeEnabled",
							set = "ToggleOfflineTime",
							order = 3,
						},
					},
				},
				timezone = {
					type = "range",
					name = L["console.command.datetime.timezone.name"],
					desc = L["console.command.datetime.timezone.description"],
					get = "GetTimezone",
					set = "SetTimezone",
					min = -10 ,
					max = 14,
					step = 1,
					order = 3,
				},
				timeformat = {
					type = "toggle",
					name = L["console.command.datetime.timeformat.name"],
					desc = L["console.command.datetime.timeformat.description"],
					get = "IsTimeFormatEnabled",
					set = "ToggleTimeFormat",
					order = 4,
				},
			},
		},
		boss = {
			type = "group",
			name = L["console.command.boss.name"],
			desc = L["console.command.boss.description"],
			order = 4,
			args = {
				["add"] = {
					type = "input",
					name = L["console.command.boss.add.name"],
					desc = L["console.command.boss.add.description"],
					usage = L["console.command.boss.add.usage"],
					get = false,
					set = "AddBoss",
					guiHidden = true,
					order = 1,
				},
				groupsetup = {
					type = "group",
					name = L["console.command.boss.groupsetup.name"],
					desc = L["console.command.boss.groupsetup.description"],
					order = 2,
					args = {
						raidlistgroup = {
							type = "toggle",
							name = L["console.command.boss.groupsetup.raidlistgroup.name"],
							desc = L["console.command.boss.groupsetup.raidlistgroup.description"],
							get = "IsBossRaidListGroupEnabled",
							set = "ToggleBossRaidListGroup",
							order = 1,
						},
						waitlistgroup = {
							type = "toggle",
							name = L["console.command.boss.groupsetup.waitlistgroup.name"],
							desc = L["console.command.boss.groupsetup.waitlistgroup.description"],
							get = "IsBossWaitListGroupEnabled",
							set = "ToggleBossWaitListGroup",
							order = 2,
						},
						nolistgroup = {
							type = "toggle",
							name = L["console.command.boss.groupsetup.nolistgroup.name"],
							desc = L["console.command.boss.groupsetup.nolistgroup.description"],
							get = "IsBossNoListGroupEnabled",
							set = "ToggleBossNoListGroup",
							order = 3,
						},
						waitlist = {
							type = "toggle",
							name = L["console.command.boss.groupsetup.waitlist.name"],
							desc = L["console.command.boss.groupsetup.waitlist.description"],
							get = "IsBossWaitListEnabled",
							set = "ToggleBossWaitList",
							order = 4,

						},
					},
				},
			},
		},
		loot = {
			type = "group",
			name = L["console.command.loot.name"],
			desc = L["console.command.loot.description"],
			order = 5,
		    args = {
				minimum = {
					type = "input",
					name = L["console.command.loot.minimum.name"],
					desc = L["console.command.loot.minimum.description"],
					usage = L["console.usage.item.quality"],
					get = "GetMinimumLootQuality",
					set = "SetMinimumLootQuality",
					validate = L.itemQuality,
					order = 1,
				},
				popup = {
					type = "toggle",
					name = L["console.command.loot.popup.name"],
					desc = L["console.command.loot.popup.description"],
					get = "IsLootPopupEnabled",
					set = "ToggleLootPopup",
					order = 2,
				},
				exclude = {
					type = "execute",
					name = L["console.command.loot.exclude.name"],
					desc = L["console.command.loot.exclude.description"],
					func = "ManageExclusionList",
					order = 3,
				},
				["add"] = {
					type = "input",
					name = L["console.command.loot.add.name"],
					desc = L["console.command.loot.add.description"],
					usage = L["console.command.loot.add.usage"],
					get = false,
					set = "AddLoot",
					validate = "ValidateAddLoot",
					guiHidden = true,
					order = 4,
				},
			},
		},
		export = {
			type = "group",
			name = L["console.command.export.name"],
			desc = L["console.command.export.description"],
			order = 6,
			args = {
				["format"] = {
					type = "input",
					name = L["console.command.export.format.name"],
					desc = L["console.command.export.format.description"],
					usage = L["console.command.export.format.usage"],
					get = "GetExportFormat",
					set = "SetExportFormat",
					--validate = exportFormats,
					order = 1,
				},
				["eqdkp"] = {
					type = "group",
					name = L["console.command.export.eqdkp.name"],
					desc = L["console.command.export.eqdkp.description"],
					order = 2,
					args = {
						["difficulty"] = {
							type = "toggle",
							name = L["console.command.export.eqdkp.difficulty.name"],
							desc = L["console.command.export.eqdkp.difficulty.description"],
							get = "IsEQDKPDifficultyEnabled",
							set = "ToggleEQDKPDifficulty",
							order = 1,
						},
					},
				},
			},
		},
		reporting = {
			type = "group",
			name = L["console.command.reporting.name"],
			desc = L["console.command.reporting.description"],
			order = 7,
			args = {
				broadcastchannel = {
					type = "input",
					name = L["console.command.reporting.broadcastchannel.name"],
					desc = L["console.command.reporting.broadcastchannel.description"],
					usage = L["console.usage.channels"],
					get = "GetBroadcastChannel",
					set = "SetBroadcastChannel",
					--validate = channels,
					order = 1,
				},
				bosskills = {
					type = "toggle",
					name = L["console.command.reporting.bosskills.name"],
					desc = L["console.command.reporting.bosskills.description"],
					get = "IsBossBroadcastEnabled",
					set = "ToggleBossBroadcast",
					order = 2,
				},
				loot = {
					type = "toggle",
					name = L["console.command.reporting.loot.name"],
					desc = L["console.command.reporting.loot.description"],
					get = "IsLootBroadcastEnabled",
					set = "ToggleLootBroadcast",
					order = 3,
				},
			},
		},
        ["debug"] = {
            type = "toggle",
            name = L["console.command.debug.name"],
            desc = L["console.command.debug.description"],
            get = "IsDebugEnabled",
            set = "ToggleDebugEnabled",
			order = 8,
        },
		exclude = {
			type = "group",
			name = L["console.command.exclude.name"],
			desc = L["console.command.exclude.description"],
			guiHidden = true,
			order = 9,
			args = {
				add = {
					type = "input",
					name = L["console.command.exclude.add.name"],
					desc = L["console.command.exclude.add.description"],
					usage = L["console.usage.exclude"],
					get = false,
					set = "AddToExclusionList",
					validate = "ValidateAddToExclusionList",
					order = 1,
				},
				["remove"] = {
					type = "input",
					name = L["console.command.exclude.remove.name"],
					desc = L["console.command.exclude.remove.description"],
					usage = L["console.usage.exclude"],
					get = false,
					set = "RemoveFromExclusionList",
					validate = "ValidateRemoveFromExclusionList",
					order = 2,
				},
		        ["list"] = {
		            type = "execute",
		            name = L["console.command.exclude.list.name"],
		            desc = L["console.command.exclude.list.description"],
					func = "DisplayExclusionList",
					order = 3,
		        },
			},
		},
    },
}


local defaults = {
	-- Group Management
	raidListGroups = {
		["1"] = true,
		["2"] = true,
		["3"] = true,
		["4"] = true,
		["5"] = true,
		["6"] = false,
		["7"] = false,
		["8"] = false,
	},
	waitListGroups = {
		["1"] = false,
		["2"] = false,
		["3"] = false,
		["4"] = false,
		["5"] = false,
		["6"] = true,
		["7"] = true,
		["8"] = true,
	},
	-- raid management
	raid = {
		["groupsetup"] = {
			["isAutomaticGroupSelectionEnabled"] = true,
		},
		["waitlist"] = {
			["duration"] = 0, 		-- default announce duration of always enabled
			["autoremoval"] = true, -- automatically remove players from the wait list upon joining a raid list group
			["incoming"] = true, 	-- display incoming wait list whispers
			["outgoing"] = false, 	-- do NOT display outgoing wait list whispers
			["notify"] = false, 	-- do NOT display the waitlist notification message
		},
		["pruning"] = {
			["isPruningEnabled"] = false,
			["pruningTime"] = 12,	-- default pruning time of ~3 months
		},
	},
	-- boss management
	boss = {
		["groupsetup"] = {
			["raidlistgroup"] = true,
			["waitlistgroup"] = false,
			["nolistgroup"] = false,
			["waitlist"] = false,
		},
	},
	delay = 5,	-- default delay value of 5 seconds
	isBattlegroundTrackingEnabled = false,
	dateFormat = 2,
	timezone = -5, -- "-5 GMT, Eastern Standard Time"
	isTimeFormatEnabled = true,
	isRaidListTimeEnabled = true,
	isWaitListTimeEnabled = false,
	isOfflineTimeEnabled = false,
	minimumLootQuality = 4,
	isLootPopupEnabled = false,
	broadcastChannel = channels["guild"],
	isLootBroadcastEnabled = false,
	isBossBroadcastEnabled = false,
	exportFormat = "EQdkp",
	-- export
	export = {
		["eqdkp"] = {
			["difficulty"] = true,
		},
	},
	isDebugEnabled = false,
	raidListWrapper = nil,
	exclusionList = {
		[20725] = true, -- Nexus Crystal
		[22450] = true, -- Void Crystal
		[30311] = true, -- Warp Slicer
		[30312] = true, -- Infinity Blade
		[30313] = true, -- Staff of Disintegration
		[30314] = true, -- Phaseshift Bulwark
		[30316] = true, -- Devastation
		[30317] = true, -- Cosmic Infuser
		[30318] = true, -- Netherstrand Longbow
		[30319] = true, -- Nether Spike
		[30320] = true, -- Bundle of Nether Spikes
		[34057] = true, -- Abyss Crystal
		[36919] = true, -- Cardinal Ruby
		[36922] = true, -- King's Amber
		[36925] = true, -- Majestic Zircon
		[36928] = true, -- Dreadstone
		[36931] = true, -- Ametrine
		[36934] = true, -- Eye of Zul
		[49110] = true, -- Nightmare Tear
	},
}

function HeadCount:GetDBDefaults()
	return { profile = defaults }
end

function HeadCount:GetExportFormat()
	return self.db.profile.exportFormat
end

function HeadCount:SetupOptions()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("HeadCount2", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HeadCount2", "HeadCount2")
	--self:RegisterChatCommand("hc2", function() LibStub("AceConfigDialog-3.0"):Open("HeadCount2") end )
end
